/*  Board.cpp: Board functions
 *
 *  Feringa is a chess game for the Palm Pilot based on OpenChess.
 *  Copyright (C) 2005, Manfred Joerg.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "Board.hxx"
#include "OpenChess_res.h"
#include "gnuchess2.h"
#include "OpenChess.h"

#include <Bitmap.h>

#define OS35GLUE
#ifdef OS35GLUE
#include <Libraries/PalmOSGlue/BmpGlue.h>
#endif

// Parameters for the board and pieces bitmap

short offSqWhite;	// X offset of the Wh square
short szSq;		// Board size square
short offSqBlack;	// X offset of the Bl square
short offSqMark;	// X mark offset
short offSqMask;	// X Mark mask offset

short TitleHeight;
short XOffPrgBkg;
short YOffPrgBkg;
short XOffPrgMtr;
short Prg;
short Xprg;
short Yprg;

struct TsqSel sqSel;

WinHandle DispWindowH, OffscreenWinH;
Boolean  bCoordinates = false;

void ReleaseBoard()
{
   WinDeleteWindow(OffscreenWinH, false);  // Delete offscreenwindow
}

void InitBoard()
{
   Err         err;
   MemHandle   resH;
   BitmapPtr   resP;
   Coord	    width, height;
   UInt32      version;

   /* Check for OS5 hi-res display.  FIrst - is Window Mgr new enough? */

  err = FtrGet(sysFtrCreator, sysFtrNumWinVersion, &version);
  resH = DmGetResource(bitmapRsc, BmpSymbols);

  // Create offscreen window, and load pieces bitmap

  resP = (BitmapPtr) MemHandleLock(resH);
#ifdef OS35GLUE
  BmpGlueGetDimensions(resP, &width, &height, NULL);
#else
  BmpGetDimensions(resP, &width, &height, NULL);
#endif

   {
    /* Lo-res (160x160) display */
    OffscreenWinH = WinCreateOffscreenWindow(width, height, screenFormat, &err);
    if (OffscreenWinH) {
      DispWindowH = WinSetDrawWindow(OffscreenWinH);
      WinDrawBitmap (resP, 0, 0);
    }

    /* Set a whole lot of constants for this screen resolution */
    offSqWhite = 0;
    szSq = 18;
    offSqBlack = offSqWhite;
    offSqMark = offSqWhite + szSq;
    offSqMask = offSqMark + szSq;

    TitleHeight = 13;
    XOffPrgBkg = 0;
    YOffPrgBkg = 2 * szSq;
    XOffPrgMtr = XOffPrgBkg + szSq;
    Prg = 10;
    Xprg = 7 * szSq ;
    Yprg = 8 * szSq + 3 ;
  }

  MemPtrUnlock(resP);
  DmReleaseResource(resH);
  WinSetDrawWindow(DispWindowH);          // Restore the current draw window
}

/****************************************************************************/
void Copy2screen(Coord xs, Coord ys, Coord ex, Coord ey,
                 Coord xd, Coord yd, WinDrawOperation mode)
{
  RectangleType   rect;
  rect.topLeft.x = xs;
  rect.topLeft.y = ys;
  rect.extent.x = ex;
  rect.extent.y = ey;

  WinCopyRectangle(OffscreenWinH, DispWindowH, &rect, xd, yd, mode);
}

void DrawPieceOnly(const UInt8 uiColor, const UInt8 uiPiece, const UInt8 x, const UInt8 y)
{
   MemHandle resH;
   BitmapType *bitmap;

   if (uiColor == white)
   	resH = DmGetResource(bitmapRsc, BmpWPawn + uiPiece);
   else
   	resH = DmGetResource(bitmapRsc, BmpBPawn + uiPiece);
	bitmap = (BitmapType*) MemHandleLock(resH);
   WinSetDrawMode(winPaint);
	WinDrawBitmap(bitmap, x, y);
   MemHandleUnlock(resH);
   DmReleaseResource(resH);
}

/****************************************************************************/
void DrawPiece(Int16 sq)
{
short r,c, offset;
Coord x, y;

  if (sq < 0) return;

  if (reverse) r = 7-row[sq]; else r = row[sq];
  if (reverse) c = 7-column[sq]; else c = column[sq];

  r = 7-r;

  offset = ((c + r) % 2) ? szSq : 0;      // Draw the board square
  x = c * szSq; y = r * szSq;
  Copy2screen(offSqWhite, offset, szSq, szSq, x, y, winPaint);

  if (sqSel.start == sq) {    // Display the start selection mark
    Copy2screen(offSqMask, 0, szSq, szSq, x, y, winMask);
    Copy2screen(offSqMark, szSq, szSq, szSq, x, y, winOverlay);
    }

  if (sqSel.end == sq) {    // Display the end selection mark
    Copy2screen(offSqMask, 0, szSq, szSq, x, y, winMask);
    Copy2screen(offSqMark, 0, szSq, szSq, x, y, winOverlay);
    }

  if (board[sq] > 0)    // Draw the piece
  {
    x = c * szSq + 1;
    y = r * szSq + 1;
    DrawPieceOnly(color[sq], board[sq] - 1, x, y);

    }
	 else if (bCoordinates == true)
	 {
	   Char Coordinates[2];
		r = 7 - r;
      if (reverse)
		{
         Coordinates[0] = 'h' - c;
		   Coordinates[1] = '8' - r;
		}
		else
		{
		   Coordinates[0] = 'a' + c;
		   Coordinates[1] = '1' + r;
		}
      WinSetDrawMode(winOverlay);
		WinPaintChars(Coordinates, 2, 6 + c * 18, 130 - 18 * r);
	 }
}


/****************************************************************************/
void UpdateDisplay(short f, short t, short flag, short iscastle)    // adapted from uxdsp.c
{
UInt16    i;

  if (flag) for (i = 0; i < 64; i++) {
    DrawPiece(i);
    }
  else {
    DrawPiece(f); DrawPiece(t);
    if (iscastle) {
      if (t > f)  { DrawPiece(f + 3); DrawPiece(t - 1); }
      else        { DrawPiece(f - 4); DrawPiece(t + 1); }
      }
    }
}

/****************************************************************************/
Boolean getsq(Int16 ScreenX, Int16 ScreenY, UInt16 *sq)
{
  if ((ScreenX < 8 * szSq) && (ScreenY < 8 * szSq)) {
    *sq = 8 * (7 - (ScreenY / szSq)) + (ScreenX / szSq);
    if (reverse) *sq = 63 - *sq;
    return true;
    }
  else return false;
}

/****************************************************************************/
void UnselectPieces(void)
{
Int16   f, t;

  f = sqSel.start; t = sqSel.end;
  sqSel.start = sqSel.end = NoSqSelected;
  DrawPiece(f); DrawPiece(t);
}